from django_query_prefixer import default_prefixer_fn, set_prefix


def test_default_prefixer_fn():
    set_prefix("test1", "value1")
    set_prefix("test2", "value2")

    assert default_prefixer_fn() == "test1=value1 test2=value2"



from unittest import mock

from django_query_prefixer.middlewares import request_route


def test_request_route_middleware():
    response = object()
    get_response = mock.MagicMock(return_value=response)
    middleware = request_route(get_response=get_response)

    request = mock.MagicMock()
    request.resolver_match.route = "/hello"

    def hello_world():
        pass

    with mock.patch("django_query_prefixer.middlewares.set_prefix") as mock_set_prefix, mock.patch(
        "django_query_prefixer.middlewares.remove_prefix"
    ) as mock_remove_prefix:
        middleware.process_view(request, hello_world, [], {})
        assert middleware(request) == response

        mock_set_prefix.assert_has_calls(
            [
                mock.call(key="view_name", value=f"test_middlewares.hello_world"),
                mock.call(key="route", value=f"/hello"),
            ]
        )
        mock_remove_prefix.assert_has_calls(
            [
                mock.call("view_name"),
                mock.call("route"),
            ],
        )



from contextlib import contextmanager
from typing import Dict
from contextvars import ContextVar

__version__ = "0.1.3"


@contextmanager
def sql_prefixes(**kwargs):
    for key, value in kwargs.items():
        if not isinstance(value, str):
            value = repr(value)
        set_prefix(key, value)

    yield

    for key in kwargs.keys():
        remove_prefix(key)


def set_prefix(key: str, value: str):
    prefixes = _prefixes.get({})
    prefixes[key] = value
    _prefixes.set(prefixes)


def get_prefixes() -> Dict[str, str]:
    return _prefixes.get({})


def remove_prefix(key: str):
    prefixes = _prefixes.get({})
    prefixes.pop(key, None)
    _prefixes.set(prefixes)


def default_prefixer_fn() -> str:
    prefixes = get_prefixes()
    return " ".join(f"{key}={value}" for key, value in prefixes.items())


_prefixes: ContextVar[Dict[str, str]] = ContextVar("sql_prefixes")



from django_query_prefixer import remove_prefix, set_prefix


class RequestRouteMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        remove_prefix("view_name")
        remove_prefix("route")
        return response

    def process_view(self, request, view_func, view_args, view_kwargs):
        set_prefix(key="view_name", value=f"{view_func.__module__}.{view_func.__name__}")
        set_prefix(key="route", value=escape_comment_markers(request.resolver_match.route))


def request_route(get_response):
    return RequestRouteMiddleware(get_response)


def escape_comment_markers(value):
    return value.replace("/*", r"\/\*").replace("*/", r"\*\/")



from django.conf import settings
from django.utils.module_loading import import_string


class CursorWrapper:
    def __init__(self, cursor):
        self.cursor = cursor

    def execute(self, sql, params=None):
        prefix = self.generate_prefix()
        sql = f"/* {prefix} */\n{sql}"
        return self.cursor.execute(sql, params)

    def executemany(self, sql, param_list):
        prefix = self.generate_prefix()
        sql = f"/* {prefix} */\n{sql}"
        return self.cursor.executemany(sql, param_list)

    def generate_prefix(self):
        prefixer_fn = getattr(settings, "DJANGO_QUERY_PREFIXER_FUNCTION", "django_query_prefixer.default_prefixer_fn")
        fn = import_string(prefixer_fn)
        return fn()

    def __getattr__(self, attr):
        if attr in self.__dict__:
            return self.__dict__[attr]
        return getattr(self.cursor, attr)

    def __iter__(self):
        return iter(self.cursor)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()



from django_query_prefixer import get_prefixes, set_prefix, sql_prefixes


def test_sql_prefixes_context_manager():
    with sql_prefixes(test="dummy", n=42):
        assert get_prefixes() == {"test": "dummy", "n": "42"}
    assert get_prefixes() == {}


def test_set_prefixes():
    set_prefix("key1", "value1")

    assert get_prefixes() == {"key1": "value1"}


def test_set_prefixes_override_previous_value():
    set_prefix("key1", "value1")
    set_prefix("key1", "value2")

    assert get_prefixes() == {"key1": "value2"}


def test_set_prefixes_multiple():
    set_prefix("key1", "value1")
    set_prefix("key2", "value2")

    assert get_prefixes() == {
        "key1": "value1",
        "key2": "value2",
    }



from django.db.backends.sqlite3.base import DatabaseWrapper as BaseDatabaseWrapper

from ...cursor import CursorWrapper


class DatabaseWrapper(BaseDatabaseWrapper):
    def create_cursor(self, name=None):
        cursor = super().create_cursor(name)
        return CursorWrapper(cursor)
